/* Copyright (c) 2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.model;

import se.llbit.chunky.world.Texture;
import se.llbit.math.AABB;
import se.llbit.math.Ray;

/**
 * A lava block. The height the top lava block is slightly lower
 * than a regular block.
 * @author Jesper Öqvist <jesper@llbit.se>
 */
public class LavaModel {
	private static AABB fullBlock = new AABB(0, 1, 0, 1, 0, 1);
	private static AABB[] block = {
		new AABB(0, 1, 0, .875, 0, 1),
		new AABB(0, 1, 0, 14/16., 0, 1),
		new AABB(0, 1, 0, 12/16., 0, 1),
		new AABB(0, 1, 0, 10/16., 0, 1),
		new AABB(0, 1, 0, 8/16., 0, 1),
		new AABB(0, 1, 0, 6/16., 0, 1),
		new AABB(0, 1, 0, 4/16., 0, 1),
		new AABB(0, 1, 0, 2/16., 0, 1),
	};

	public static boolean intersect(Ray ray) {
		ray.t = Double.POSITIVE_INFINITY;
		int lavaAbove = (ray.currentMaterial >> 16) & 1;
		int level = (ray.currentMaterial >> 8) % 8;
		
		if (lavaAbove != 0) {
			if (fullBlock.intersect(ray)) {
				Texture.lava.getColor(ray);
				ray.distance += ray.tNear;
				ray.x.scaleAdd(ray.tNear, ray.d, ray.x);
				return true;
			}
			return false;
		}
		
		if (block[level].intersect(ray)) {
			Texture.lava.getColor(ray);
			ray.color.w = 1;
			ray.distance += ray.tNear;
			ray.x.scaleAdd(ray.tNear, ray.d, ray.x);
			return true;
		}
		return false;
	}
}
