/* Copyright (c) 2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.model;

import se.llbit.chunky.world.Texture;
import se.llbit.math.AABB;
import se.llbit.math.Quad;
import se.llbit.math.Ray;
import se.llbit.math.Vector3d;
import se.llbit.math.Vector4d;

/**
 * A lever switch.
 * @author Jesper Öqvist <jesper@llbit.se>
 */
public class LeverModel {
	private static AABB block =
		new AABB(.25, .75, 0, .1875, .3125, .6875);

	private static Quad[] lever = {
		new Quad(new Vector3d(.5625, 0, .4375), new Vector3d(.4375, 0, .4375),
				new Vector3d(.5625, .625, .4375), new Vector4d(.5625, .4375, 0, .625)),

		new Quad(new Vector3d(.4375, 0, .5625), new Vector3d(.5625, 0, .5625),
				new Vector3d(.4375, .625, .5625), new Vector4d(.4375, .5625, 0, .625)),
				
		new Quad(new Vector3d(.4375, 0, .4375), new Vector3d(.4375, 0, .5625),
				new Vector3d(.4375, .625, .4375), new Vector4d(.4375, .5625, 0, .625)),

		new Quad(new Vector3d(.5625, 0, .5625), new Vector3d(.5625, 0, .4375),
				new Vector3d(.5625, .625, .5625), new Vector4d(.5625, .4375, 0, .625)),
				
		// top
		new Quad(new Vector3d(.4375, .625, .5625), new Vector3d(.5625, .625, .5625),
				new Vector3d(.4375, .625, .4375), new Vector4d(.4375, .5625, .5, .625)),
	};

	private static Quad[][] rot = new Quad[2][];

	static {
		// pointing east
		rot[0] = new Quad[lever.length];
		for (int i = 0; i < lever.length; ++i) {
			rot[0][i] = lever[i].getZRotated(- Math.PI / 4);
		}
		// pointing west
		rot[1] = new Quad[lever.length];
		for (int i = 0; i < lever.length; ++i) {
			rot[1][i] = lever[i].getZRotated(Math.PI / 4);
		}
	}

	public static boolean intersect(Ray ray) {
		boolean hit = false;
		ray.t = Double.POSITIVE_INFINITY;
		if (block.intersect(ray)) {
			Texture.cobblestone.getColor(ray);
			ray.t = ray.tNear;
			hit = true;
		}
		for (Quad quad : rot[0]) {
			if (quad.intersect(ray)) {
				float[] color = Texture.lever.getColor(ray.u, ray.v);
				if (color[3] > Ray.EPSILON) {
					ray.color.set(color);
					ray.n.set(quad.n);
					ray.t = ray.tNear;
					hit = true;
				}
			}
		}
		if (hit) {
			ray.color.w = 1;
			ray.distance += ray.t;
			ray.x.scaleAdd(ray.t, ray.d, ray.x);
		}
		return hit;
	}
}
