/* Copyright (c) 2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.model;

import se.llbit.chunky.world.Texture;
import se.llbit.math.Quad;
import se.llbit.math.Ray;
import se.llbit.math.Vector3d;
import se.llbit.math.Vector4d;

/**
 * A generic block.
 * @author Jesper Öqvist <jesper@llbit.se>
 */
public class SignPostModel {
	protected static Quad[] sides = {
		// front
		new Quad(new Vector3d(.5625, .5, 0), new Vector3d(.4375, .5, 0),
				new Vector3d(.5625, 1, 0), new Vector4d(.5625, .4375, 0, 1)),

		// back
		new Quad(new Vector3d(.4375, .5, 1), new Vector3d(.5625, .5, 1),
				new Vector3d(.4375, 1, 1), new Vector4d(.4375, .5625, 0, 1)),
		
		// right
		new Quad(new Vector3d(.4375, .5, 0), new Vector3d(.4375, .5, 1),
				new Vector3d(.4375, 1, 0), new Vector4d(0, 1, 0, 1)),

		// left
		new Quad(new Vector3d(.5625, .5, 1), new Vector3d(.5625, .5, 0),
				new Vector3d(.5625, 1, 1), new Vector4d(1, 0, 0, 1)),

		// top
		new Quad(new Vector3d(.5625, 1, 0), new Vector3d(.4375, 1, 0),
				new Vector3d(.5625, 1, 1), new Vector4d(.5625, .4375, 0, 1)),
		
		// bottom
		new Quad(new Vector3d(.4375, .5, 0), new Vector3d(.5625, .5, 0),
				new Vector3d(.4375, .5, 1), new Vector4d(.4375, .5625, 0, 1)),
				
	};

	public static boolean intersect(Ray ray) {
		boolean hit = false;
		ray.t = Double.POSITIVE_INFINITY;
		for (Quad side : sides) {
			if (side.intersect(ray)) {
				Texture.oakPlanks.getColor(ray);
				ray.n.set(side.n);
				ray.t = ray.tNear;
				hit = true;
			}
		}
		if (hit) {
			ray.color.w = 1;
			ray.distance += ray.t;
			ray.x.scaleAdd(ray.t, ray.d, ray.x);
		}
		return hit;
	}
}
