/* Copyright (c) 2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.math;

public class Vector3d {
	public double x, y, z;

	public Vector3d() {
		this(0, 0, 0);
	}
	
	public Vector3d(double i, double j, double k) {
		x = i;
		y = j;
		z = k;
	}

	public final void set(Vector3d o) {
		x = o.x;
		y = o.y;
		z = o.z;
	}
	
	public final void set(double d, double e, double f) {
		x = d;
		y = e;
		z = f;
	}

	public final double dot(Vector3d o) {
		return x*o.x + y*o.y + z*o.z;
	}

	public final void sub(Vector3d a, Vector3d b) {
		x = a.x - b.x;
		y = a.y - b.y;
		z = a.z - b.z;
	}

	public final double lengthSquared() {
		return x*x + y*y + z*z;
	}

	public final void cross(Vector3d a, Vector3d b) {
		x = a.y*b.z - a.z*b.y;
		y = a.z*b.x - a.x*b.z;
		z = a.x*b.y - a.y*b.x;
	}

	public final void normalize() {
		double s = 1/Math.sqrt(lengthSquared());
		x *= s;
		y *= s;
		z *= s;
	}

	public final void scaleAdd(double s, Vector3d d, Vector3d o) {
		x = s*d.x + o.x;
		y = s*d.y + o.y;
		z = s*d.z + o.z;
	}

	public final void scale(double s) {
		x *= s;
		y *= s;
		z *= s;
	}

	public final void add(Vector3d a, Vector3d b) {
		x = a.x + b.x;
		y = a.y + b.y;
		z = a.z + b.z;
	}

	public final void add(Vector3d a) {
		x += a.x;
		y += a.y;
		z += a.z;
	}

	public final void sub(Vector3d a) {
		x -= a.x;
		y -= a.y;
		z -= a.z;
	}

	public void set(Vector3i a) {
		x = a.x;
		y = a.y;
		z = a.z;
	}
}
